from copy import deepcopy
from typing import List
import pickle

import numpy as np

import serpentTools as st

from krakentools.groupconstants.genpoly.records.materials import HomogenizedMaterial, SelectMaterial
from krakentools.groupconstants.genpoly.genpoly import GenPoly
from krakentools.groupconstants.genpoly.definitions import FitVariables
from krakentools.containers import condense_node_branches
from krakentools.serpent import read_all_gc_results, unfold_symmetric_ppr_data, parse_mdep_dec

from krakentools.serpentdefs import GCTypes, DCTypes
from krakentools.groupconstants.genpoly.records.pins import PinTable

st.settings.rc["verbosity"] = "critical"

# --- Number of processes for parallel tasks

n_proc = 40

################################################################################

# --- Group condensation

#condensation = (0,1,2,3,4,5,6,7) # Original 8 groups
condensation = (0,0,0,0,1,1,1,1) # Two groups

# --- Number of groups for the parametrization

ng = max(condensation) + 1

################################################################################

# --- Each assembly lattice had the same symmetry

sym_str = "set usym lA6s 3 2 0.0 0.0 270 45"

# --- Set the nominal point and nominal history point
#     The Serpent results are sorted to nominals and variations based on these
#     definitiions

nom_point = np.zeros((len(FitVariables)))
nom_point[FitVariables.TFU.value] = 561
nom_point[FitVariables.TCO.value] = 404.15
nom_point[FitVariables.DCO.value] = 0.9342

# --- The nominal history point could be different from the nominal branch point

nom_his_point = nom_point.copy()

##############################################
# --- Set up the fit polynomial (four terms) #
##############################################

my_fit = np.zeros((4, len(FitVariables)))

# --- First and second degree terms for TFUSQRT

my_fit[0, FitVariables.TFUSQRT] = 1
my_fit[1, FitVariables.TFUSQRT] = 2

# --- First and second degree terms for DCO

my_fit[2, FitVariables.DCO] = 1
my_fit[3, FitVariables.DCO] = 2

############################################################################################
############################################################################################

# --- Parse decay constants of the plutonium chain from microdepletion output

mdx_path = "./fuel/A1_nominals_h1_r1_mdx1b1.m"
dec_dict = parse_mdep_dec(mdx_path)

# --- Generate a single parametrization object into which the different assemblies will be added

my_parametrization = GenPoly(dec_data_dict=dec_dict)

############################################################################################
############################################################################################

# --- Loop over all assembly types and add their data to the parametrization

for assembly in range(1,7):

    # --- Get all results for this assembly type using parallel read from disk

    node_branch_list = read_all_gc_results(f"./fuel/A{assembly}", n_proc=n_proc)

    # --- Condense the data in parallel (parallel is not much faster than serial here for some reason)

    if ng < 8:
        node_branch_list = condense_node_branches(node_branch_list, condensation, n_proc=n_proc)

    # --- Get uranium mass per cm from one of the NodeBranch objects
    #     Multiply by 4 due to calculation being done for a quarter assembly
    #     Here we would need to divide by model height if Serpent model was something else than 1 cm high
    #     Convert from kg/cm to g/cm

    u_mass_per_cm = node_branch_list[0].hg_uni.gc["iniFmass"][0]*4*1000.0

    # --- Unfold pinpower reconstruction data from symmetry sector

    print("Unfolding...")
    unfold_symmetric_ppr_data(node_branch_list, sym_str, is_quarter=True, height=1, remove_nonsymmetric=True, remove_zeropower=True)

    # --- Create pin table based on data in node branches
    #     (gd pins really have lower U content)

    u_mass_per_cm_pins = u_mass_per_cm/264.0
    pin_table = PinTable.from_nodebranch(node_branch_list[0], u_mass_per_cm_pins)

    # --- Create a material out of this

    new_mat = HomogenizedMaterial(f"fuel{assembly}", u_mass_per_cm, nom_point, fit=my_fit,
                                    is_quarter=True, nominal_his_point=nom_his_point,
                                    description=f"Fuel assembly type {assembly}")

    # --- Add this data to the material representing this assembly type

    new_mat.add_data(node_branch_list)
    new_mat.add_pin_table(pin_table)

    # --- Add material to the parametrization

    my_parametrization.add_material(new_mat)

# --- Apply the parametrization (leakage corrected data and leakage corrected diffusion coefficient)

my_parametrization.parametrize(gc_type=GCTypes.B1, diff_coef_type=DCTypes.B1, version=3)

# --- Write the parametrization to a file

fout = open(f"./gc/fuel_{ng}g_lc.xs", "w")
my_parametrization.write_to_file(fout)
fout.close()

# --- Apply the parametrization (infinite spectrum data with CMM diffusion coefficient)

my_parametrization.parametrize(gc_type=GCTypes.INF, diff_coef_type=DCTypes.CMM, version=3)

# --- Write the parametrization to a file

fout = open(f"./gc/fuel_{ng}g_nc.xs", "w")
my_parametrization.write_to_file(fout)
fout.close()