import numpy as np

from krakentools.groupconstants.genpoly.definitions import FitVariables
from krakentools.CoreLattice import CoreLattice
from krakentools.serpent import read_all_gc_results

n_groups = 2

if n_groups == 8:
    condensation_list = None
elif n_groups == 4:
    condensation_list = [0, 1, 2, 2, 3, 3, 3, 3]
elif n_groups == 2:
    condensation_list = [0, 0, 0, 0, 1, 1, 1, 1]
else:
    raise RuntimeError("Incorrect number of groups")

###############################################################
# --- Creates surfaces, universes, set gcu and set adf cards ##
#     for reflector homogenization                           ##
###############################################################

# --- Core lattice definition from Serpent input
#     Lattice type, size, pitch and origin are taken from here

core_lattice = "lat c1.1  1  0.0 0.0 9 9  21.50364"


# --- Core loading pattern (lattice elements) from Serpent input

core_loading = """XX XX XX XX XX XX XX XX XX
                  XX XX XX XX XX XX XX XX XX
                  XX XX XX XX XX XX A5 XX XX
                  XX XX XX XX XX A2 A3 A4 XX
                  XX XX XX XX A1 A2 A2 A6 XX
                  XX XX XX XX XX XX XX XX XX
                  XX XX XX XX XX XX XX XX XX
                  XX XX XX XX XX XX XX XX XX
                  XX XX XX XX XX XX XX XX XX"""

# --- Create dictionaries linking assemblies to their infinite lattice .coe
#     and _res.m files

assemblies = ["A4", "A5", "A6"]

# --- List of strings indicating an empty (non-fueled) position in the
#     loading pattern

core_empty_pos_names = ["XX"]

# --- Core symmetry card from Serpent input or an empty string literal: ""
#     The symmetry of the case is taken from here

core_symmetry = "set usym core 3 2 0.0 0.0 315 45"

##############################################################################
##############################################################################
##############################################################################

### Create a separate object

cl_object = CoreLattice()

### Parse the data

cl_object.build_model(core_lattice,
                      core_loading,
                      core_empty_pos_names,
                      core_symmetry,
                      subdiv_xy=2,
                      zmin=60.0, zmax=90.0)

print(cl_object)

##############################################################################
### Link infinite lattice results ############################################
##############################################################################

node_branch_dict = {}

for ass_name in assemblies:
    big_node_branch_list = read_all_gc_results(f"./fuel/{ass_name}", n_proc=40)

    # --- Throw away rodded and spacered infinite lattice data

    node_branch_list = []
    for node_branch in big_node_branch_list:
        if node_branch.var_dict["cr"] == "0" and node_branch.var_dict["spa"] == "0" and node_branch.hg_uni.bu == 0:
            # --- Condense if needed

            if condensation_list is not None:
                new_nb = node_branch.give_condensed_version(condensation_list)
            else:
                new_nb = node_branch

            node_branch_list.append(new_nb)

    # --- We need to provide data for four corners, but the same data works for all of them

    for corner in range(1, 5):
        ass_key = "{}_{}".format(ass_name, corner)
        node_branch_dict[ass_key] = node_branch_list

        print(f"{ass_key}: {len(node_branch_dict[ass_key])} branches")

##############################################################################
### Serpent 2D full core simulation ##########################################
##############################################################################

node_branches = read_all_gc_results("./reflector_radial/Serpent.inp")

# --- Condense if needed

if condensation_list:
    new_branches = []
    for nb in node_branches:
        new_branches.append(nb.give_condensed_version(condensation_list))
    node_branches = new_branches

# --- Link node branches to core lattice (also symmetricizes the results)

cl_object.link_radial_reflector_data(node_branches)

# --- Link infinite lattice data

cl_object.link_inf_lattice_data(node_branch_dict)

### Create some output

print(cl_object)

##############################################################################
### Ants 2D single node simulations ##########################################
##############################################################################

cl_object.solve_ants_nodes(n_proc=40, n_iter_max=200)

##############################################################################
### Corrected DF calculation and writing  ####################################
##############################################################################

# --- Choose the fit to apply

my_fit = np.zeros((2, len(FitVariables)))
my_fit[0, FitVariables.DCO.value] = 1     # --- First order term for coolant density
my_fit[1, FitVariables.DCO.value] = 2     # --- Second order term for coolant density

# --- Set the nominal point (everything is zero by default)
#     The Serpent results are sorted to nominals and variations based on these
#     definitiions

nom_point = np.zeros((len(FitVariables)))

# --- Set some nominal values

nom_point[FitVariables.TFU.value] = 561.0
nom_point[FitVariables.TCO.value] = 404.15
nom_point[FitVariables.DCO.value] = 0.9342
nom_point[FitVariables.BOR.value] = 0.0

# ---

cl_object.calculate_reflector_dfs_from_data(corrected=False)
cl_object.write_gen_poly(f"gc/{n_groups}g_poly_radial_uncorrected.xs", nom_point, fit=my_fit, version=3)

cl_object.calculate_reflector_dfs_from_data(corrected=True)
cl_object.write_gen_poly(f"gc/{n_groups}g_poly_radial_corrected.xs", nom_point, fit=my_fit, version=3)
